
;;;; Begin Copyright Notice ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Copyright (c) 2015-2021 The Khronos Group Inc.
; Copyright (c) 2015-2021 Valve Corporation
; Copyright (c) 2015-2021 LunarG, Inc.
;
; Licensed under the Apache License, Version 2.0 (the "License");
; you may not use this file except in compliance with the License.
; You may obtain a copy of the License at
;
;     http://www.apache.org/licenses/LICENSE-2.0
;
; Unless required by applicable law or agreed to in writing, software
; distributed under the License is distributed on an "AS IS" BASIS,
; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
; See the License for the specific language governing permissions and
; limitations under the License.
;
;  Author: Courtney Goeltzenleuchter <courtney@LunarG.com>
;  Author: Charles Giessen <charles@lunarg.com>
;
;;;;  End Copyright Notice ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

LIBRARY vulkan-1.dll
EXPORTS
   vkCreateInstance
   vkDestroyInstance
   vkEnumeratePhysicalDevices
   vkGetPhysicalDeviceFeatures
   vkGetPhysicalDeviceFormatProperties
   vkGetPhysicalDeviceImageFormatProperties
   vkGetPhysicalDeviceProperties
   vkGetPhysicalDeviceQueueFamilyProperties
   vkGetPhysicalDeviceMemoryProperties
   vkGetInstanceProcAddr
   vkGetDeviceProcAddr
   vkCreateDevice
   vkDestroyDevice
   vkEnumerateInstanceExtensionProperties
   vkEnumerateDeviceExtensionProperties
   vkEnumerateInstanceLayerProperties
   vkEnumerateDeviceLayerProperties
   vkGetDeviceQueue
   vkQueueSubmit
   vkQueueWaitIdle
   vkDeviceWaitIdle
   vkAllocateMemory
   vkFreeMemory
   vkMapMemory
   vkUnmapMemory
   vkFlushMappedMemoryRanges
   vkInvalidateMappedMemoryRanges
   vkGetDeviceMemoryCommitment
   vkBindBufferMemory
   vkBindImageMemory
   vkGetBufferMemoryRequirements
   vkGetImageMemoryRequirements
   vkGetImageSparseMemoryRequirements
   vkGetPhysicalDeviceSparseImageFormatProperties
   vkQueueBindSparse
   vkCreateFence
   vkDestroyFence
   vkResetFences
   vkGetFenceStatus
   vkWaitForFences
   vkCreateSemaphore
   vkDestroySemaphore
   vkCreateEvent
   vkDestroyEvent
   vkGetEventStatus
   vkSetEvent
   vkResetEvent
   vkCreateQueryPool
   vkDestroyQueryPool
   vkGetQueryPoolResults
   vkCreateBuffer
   vkDestroyBuffer
   vkCreateBufferView
   vkDestroyBufferView
   vkCreateImage
   vkDestroyImage
   vkGetImageSubresourceLayout
   vkCreateImageView
   vkDestroyImageView
   vkCreateShaderModule
   vkDestroyShaderModule
   vkCreatePipelineCache
   vkDestroyPipelineCache
   vkGetPipelineCacheData
   vkMergePipelineCaches
   vkCreateGraphicsPipelines
   vkCreateComputePipelines
   vkDestroyPipeline
   vkCreatePipelineLayout
   vkDestroyPipelineLayout
   vkCreateSampler
   vkDestroySampler
   vkCreateDescriptorSetLayout
   vkDestroyDescriptorSetLayout
   vkCreateDescriptorPool
   vkDestroyDescriptorPool
   vkResetDescriptorPool
   vkAllocateDescriptorSets
   vkFreeDescriptorSets
   vkUpdateDescriptorSets
   vkCreateFramebuffer
   vkDestroyFramebuffer
   vkCreateRenderPass
   vkDestroyRenderPass
   vkGetRenderAreaGranularity
   vkCreateCommandPool
   vkDestroyCommandPool
   vkResetCommandPool
   vkAllocateCommandBuffers
   vkFreeCommandBuffers
   vkBeginCommandBuffer
   vkEndCommandBuffer
   vkResetCommandBuffer
   vkCmdBindPipeline
   vkCmdSetViewport
   vkCmdSetScissor
   vkCmdSetLineWidth
   vkCmdSetDepthBias
   vkCmdSetBlendConstants
   vkCmdSetDepthBounds
   vkCmdSetStencilCompareMask
   vkCmdSetStencilWriteMask
   vkCmdSetStencilReference
   vkCmdBindDescriptorSets
   vkCmdBindIndexBuffer
   vkCmdBindVertexBuffers
   vkCmdDraw
   vkCmdDrawIndexed
   vkCmdDrawIndirect
   vkCmdDrawIndexedIndirect
   vkCmdDispatch
   vkCmdDispatchIndirect
   vkCmdCopyBuffer
   vkCmdCopyImage
   vkCmdBlitImage
   vkCmdCopyBufferToImage
   vkCmdCopyImageToBuffer
   vkCmdUpdateBuffer
   vkCmdFillBuffer
   vkCmdClearColorImage
   vkCmdClearDepthStencilImage
   vkCmdClearAttachments
   vkCmdResolveImage
   vkCmdSetEvent
   vkCmdResetEvent
   vkCmdWaitEvents
   vkCmdPipelineBarrier
   vkCmdBeginQuery
   vkCmdEndQuery
   vkCmdResetQueryPool
   vkCmdWriteTimestamp
   vkCmdCopyQueryPoolResults
   vkCmdPushConstants
   vkCmdBeginRenderPass
   vkCmdNextSubpass
   vkCmdEndRenderPass
   vkCmdExecuteCommands
   vkDestroySurfaceKHR
   vkGetPhysicalDeviceSurfaceSupportKHR
   vkGetPhysicalDeviceSurfaceCapabilitiesKHR
   vkGetPhysicalDeviceSurfaceFormatsKHR
   vkGetPhysicalDeviceSurfaceCapabilities2KHR
   vkGetPhysicalDeviceSurfaceFormats2KHR
   vkGetPhysicalDeviceSurfacePresentModesKHR
   vkCreateSwapchainKHR
   vkDestroySwapchainKHR
   vkGetSwapchainImagesKHR
   vkAcquireNextImageKHR
   vkQueuePresentKHR
   vkGetPhysicalDeviceDisplayPropertiesKHR
   vkGetPhysicalDeviceDisplayPlanePropertiesKHR
   vkGetDisplayPlaneSupportedDisplaysKHR
   vkGetDisplayModePropertiesKHR
   vkCreateDisplayModeKHR
   vkGetDisplayPlaneCapabilitiesKHR
   vkCreateDisplayPlaneSurfaceKHR
   vkCreateSharedSwapchainsKHR
   vkCreateWin32SurfaceKHR
   vkCreateHeadlessSurfaceEXT
   vkGetPhysicalDeviceWin32PresentationSupportKHR

   vkEnumerateInstanceVersion
   vkEnumeratePhysicalDeviceGroups
   vkGetPhysicalDeviceFeatures2
   vkGetPhysicalDeviceProperties2
   vkGetPhysicalDeviceFormatProperties2
   vkGetPhysicalDeviceQueueFamilyProperties2
   vkGetPhysicalDeviceMemoryProperties2
   vkGetPhysicalDeviceSparseImageFormatProperties2
   vkGetPhysicalDeviceExternalBufferProperties
   vkGetPhysicalDeviceExternalSemaphoreProperties
   vkGetPhysicalDeviceExternalFenceProperties
   vkBindBufferMemory2
   vkBindImageMemory2
   vkGetDeviceGroupPeerMemoryFeatures
   vkCmdSetDeviceMask
   vkCmdDispatchBase
   vkGetImageMemoryRequirements2
   vkGetBufferMemoryRequirements2
   vkTrimCommandPool
   vkGetDeviceQueue2
   vkCreateSamplerYcbcrConversion
   vkDestroySamplerYcbcrConversion
   vkGetDescriptorSetLayoutSupport
   vkGetDeviceGroupPresentCapabilitiesKHR
   vkGetDeviceGroupSurfacePresentModesKHR
   vkGetPhysicalDevicePresentRectanglesKHR
   vkAcquireNextImage2KHR
   vkCreateDescriptorUpdateTemplate
   vkDestroyDescriptorUpdateTemplate
   vkUpdateDescriptorSetWithTemplate

   vkGetPhysicalDeviceDisplayProperties2KHR
   vkGetPhysicalDeviceDisplayPlaneProperties2KHR
   vkGetDisplayModeProperties2KHR
   vkGetDisplayPlaneCapabilities2KHR

   vkGetImageSparseMemoryRequirements2
   vkGetPhysicalDeviceImageFormatProperties2

   vkCreateRenderPass2
   vkCmdBeginRenderPass2
   vkCmdNextSubpass2
   vkCmdEndRenderPass2
   vkCmdDrawIndirectCount
   vkCmdDrawIndexedIndirectCount
   vkGetSemaphoreCounterValue
   vkWaitSemaphores
   vkSignalSemaphore
   vkGetBufferDeviceAddress
   vkGetBufferOpaqueCaptureAddress
   vkGetDeviceMemoryOpaqueCaptureAddress
   vkResetQueryPool

   vkGetPhysicalDeviceToolProperties

   vkCreatePrivateDataSlot
   vkDestroyPrivateDataSlot
   vkSetPrivateData
   vkGetPrivateData
   vkCmdSetEvent2
   vkCmdResetEvent2
   vkCmdWaitEvents2
   vkCmdPipelineBarrier2
   vkCmdWriteTimestamp2
   vkQueueSubmit2
   vkCmdCopyBuffer2
   vkCmdCopyImage2
   vkCmdCopyBufferToImage2
   vkCmdCopyImageToBuffer2
   vkCmdBlitImage2
   vkCmdResolveImage2
   vkCmdBeginRendering
   vkCmdEndRendering
   vkCmdSetCullMode
   vkCmdSetFrontFace
   vkCmdSetPrimitiveTopology
   vkCmdSetViewportWithCount
   vkCmdSetScissorWithCount
   vkCmdBindVertexBuffers2
   vkCmdSetDepthTestEnable
   vkCmdSetDepthWriteEnable
   vkCmdSetDepthCompareOp
   vkCmdSetDepthBoundsTestEnable
   vkCmdSetStencilTestEnable
   vkCmdSetStencilOp
   vkCmdSetRasterizerDiscardEnable
   vkCmdSetDepthBiasEnable
   vkCmdSetPrimitiveRestartEnable
   vkGetDeviceBufferMemoryRequirements
   vkGetDeviceImageMemoryRequirements
   vkGetDeviceImageSparseMemoryRequirements